package com.oxylusflash.ui {
	import flash.display.Sprite;
	import com.oxylusflash.ui.MouseStateElement;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import com.oxylusflash.events.ScrollbarEvent;
	
	public class VerticalScrollbar extends Sprite {
		public var scrollTrack:MouseStateElement;
		public var scrollButton:MouseStateElement;
		
		private var w:Number;
		private var h:Number;
		private var sh:Number;
		private var dy:Number;
		private var sBtnY:Number = 0;
		private var drag:Boolean = false;
		
		private var _proportion:Number;
		private var _scrollPercentage:Number = 0;
		private var _targetHitZone:*;
		
		private static const PRECISION:uint = 3;
		private static const SCROLL_BUTTON_MIN_HEIGHT:Number = 10;
		
		public function VerticalScrollbar():void {
			this.removeChild(scrollTrack);
			
			scrollTrack = MouseStateElement(this.addChild(new OxyScrollTrack));
			scrollButton = MouseStateElement(this.addChild(new OxyScrollButton));
			
			this.width = scrollTrack.width * this.scaleX;
			this.height = scrollTrack.height * this.scaleY;
			
			this.proportion = 0.5;
			
			this.mouseChildren = false;
			this.buttonMode = true;
			this.useHandCursor = true;
			//this.hitArea = scrollTrack;
			this.scaleX = 1;
			this.scaleY = 1;
			
			this.addEventListener(Event.ADDED_TO_STAGE, addedToStageHandler);			
		}
		private function addedToStageHandler(event:Event):void {
			this.removeEventListener(Event.ADDED_TO_STAGE, addedToStageHandler);
			
			this.addEventListener(MouseEvent.ROLL_OVER, rollOverHandler);
			this.addEventListener(MouseEvent.ROLL_OUT, rollOutHandler);
			this.addEventListener(MouseEvent.MOUSE_DOWN, mouseDownHandler);
			stage.addEventListener(MouseEvent.MOUSE_UP, stage_mouseUpHandler);
			stage.addEventListener(MouseEvent.MOUSE_WHEEL, stage_mouseWheelHandler);
			
			this.dispatchEvent(new ScrollbarEvent(ScrollbarEvent.INIT));
		}
		
		private function rollOverHandler(event:MouseEvent):void {
			showOverState();
			
		}
		private function rollOutHandler(event:MouseEvent):void {
			hideOverState();
		}
		private function mouseDownHandler(event:MouseEvent):void {
			drag = true;
			if (scrollButton.hitTestPoint(stage.mouseX, stage.mouseY, true)) {
				dy = this.mouseY - scrollButton.y;
			} else {
				dy = Math.round(scrollButton.height / 2);
			}
			stage.addEventListener(MouseEvent.MOUSE_MOVE, stage_mouseMoveHandler);
			updateScrollButtonY(this.mouseY - dy);
		}
		private function stage_mouseUpHandler(event:MouseEvent) {
			if (drag) {
				drag = false;
				if (event.target !== this) {
					hideOverState();
				}
				stage.removeEventListener(MouseEvent.MOUSE_MOVE, stage_mouseMoveHandler);
			}
		}
		private function stage_mouseMoveHandler(event:MouseEvent):void {
			updateScrollButtonY(this.mouseY - dy);
		}
		private function stage_mouseWheelHandler(event:MouseEvent):void {
			if (event.target == _targetHitZone || event.target == this)
			//if (scrollTrack.hitTestPoint(stage.mouseX, stage.mouseY, true) || _targetHitZone == null || _targetHitZone.hitTestPoint(stage.mouseX, stage.mouseY, true)) {
				updateScrollButtonY(sBtnY - event.delta / Math.abs(event.delta) * (scrollButton.height / 2));
			//}
		}
		private function showOverState():void {
			if (!drag) {
				scrollTrack.showOverState();
				scrollButton.showOverState();
			}
		}
		private function hideOverState():void {
			if (!drag) {
				scrollTrack.hideOverState();
				scrollButton.hideOverState();
			}
		}
		private function updateScrollButtonY(sy:Number):void {
			sy = Math.round(Math.max(0, Math.min(sh, sy)));
			if (sBtnY != sy) {
				sBtnY = sy;
				scrollButton.y = sBtnY;
				_scrollPercentage = Number((sBtnY / sh).toFixed(PRECISION));
				
				this.dispatchEvent(new ScrollbarEvent(ScrollbarEvent.SCROLL, _scrollPercentage));
			}
		}
		/* Scrollbar width. */
		override public function get width():Number {
			return w;
		}
		override public function set width(value:Number):void {
			w = Math.round(value);
			scrollTrack.width = w;
			scrollButton.width = w;
		}
		/* Scrollbar height. */
		override public function get height():Number {
			return h;
		}
		override public function set height(value:Number):void {
			h = Math.round(value);
			scrollTrack.height = h;
			scrollButton.height = h;
			this.proportion = _proportion;
		}
		/* Scrollbar proportion (scroll button height / scroll track height). */
		public function get proportion():Number {
			return _proportion;
		}
		public function set proportion(value:Number):void {
			_proportion = Math.max(0, Math.min(1, Number(value.toFixed(PRECISION))));
			scrollButton.height = Math.max(SCROLL_BUTTON_MIN_HEIGHT, _proportion * h);
			sh = h - scrollButton.height;
			this.scrollPercentage = _scrollPercentage;
			
			scrollButton.visible 	= _proportion < 1;
			scrollTrack.visible 	= _proportion < 1;
		}
		/* Current scrollbar percentage. */
		public function get scrollPercentage():Number {
			return _scrollPercentage;
		}
		public function set scrollPercentage(value:Number):void {
			updateScrollButtonY(sh * Math.max(0, Math.min(1, value)));
		}
		/* Target Sprite hit zone where mouse wheel will be available. */
		public function get targetHitZone():* {
			return _targetHitZone;
		}
		public function set targetHitZone(value:*):void {
			_targetHitZone = value;
		}
	}
}